import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.metrics import mean_absolute_error, accuracy_score, cohen_kappa_score, confusion_matrix

def plot(result):
    y_pred = result[3][0]
    y_true = result[3][1]

    #plot_confusion_matrix(result, y_pred, y_true)

    mae = mean_absolute_error(y_true, y_pred)
    accuracy = accuracy_score(y_true, y_pred)
    qwk = cohen_kappa_score(y_true, y_pred, weights='quadratic')
    within_one = np.sum(np.abs(np.array(y_true) - np.array(y_pred)) <= 1) / len(y_true)

    # Print results
    print(f"Mean Absolute Error (MAE): {mae:.3f}")
    print(f"Exact Match Accuracy: {accuracy:.3%}")
    #print(f"Quadratic Weighted Kappa (QWK): {qwk:.3f}")
    print(f"Average Within-One Accuracy: {within_one}")



# Function to display multiple 1D arrays
def display_1d_arrays(array, titles=None):
    data = array

    # Compute progressive averages
    progressive_avg = np.cumsum(data) / np.arange(1, len(data) + 1)

    # Plot the progressive averages
    plt.figure(figsize=(8, 6))
    plt.plot(range(1, len(data) + 1), progressive_avg, marker='o', label='Progressive Average')
    plt.title('Progressive Averages')
    plt.xlabel('Index')
    plt.ylabel('Average')
    plt.grid(True)
    plt.legend()
    plt.show()

def plot_averages(
        list_interested_in,
        title
):
    overall_average = sum(list_interested_in) / len(list_interested_in)

    plt.scatter([1] * len(list_interested_in), list_interested_in, label="Average", alpha=0.2, color='blue')

    # Plot the overall average as a distinct point
    plt.scatter(1, overall_average, color='blue', label="Overall Average")

    # Add labels and legend
    plt.xticks([])
    plt.ylabel("Mean Absolute Error")
    #plt.ylim(0.5, 1.5)
    plt.title(title)
    plt.legend()

    # Show the plot
    plt.show()

def plot_two_averages(
        list1,
        list2,
        title1,
        title2,
        y_limits=None
):
    # Compute the overall averages and standard deviations for both lists
    overall_average1 = sum(list1) / len(list1)
    overall_average2 = sum(list2) / len(list2)
    std_dev1 = (sum((x - overall_average1) ** 2 for x in list1) / len(list1)) ** 0.5
    std_dev2 = (sum((x - overall_average2) ** 2 for x in list2) / len(list2)) ** 0.5

    # Create the figure and axes
    fig, axes = plt.subplots(1, 2, figsize=(12, 6), sharey=True)

    # Plot the first list
    axes[0].scatter([1] * len(list1), list1, label="Data Points", alpha=0.2, color='blue')
    axes[0].scatter(1, overall_average1, color='red', label="Overall Average")
    axes[0].set_title(title1)
    axes[0].set_xticks([])
    axes[0].set_ylabel("Accuracy")
    axes[0].legend()

    # Add a box with statistics for the first list
    stats_text1 = f"Average: {overall_average1:.3f}\nStd Dev: {std_dev1:.3f}"
    axes[0].text(0.7, 0.9, stats_text1, transform=axes[0].transAxes,
                 fontsize=12, verticalalignment='top', bbox=dict(boxstyle="round", facecolor="white", alpha=0.5))
    # Plot the second list
    axes[1].scatter([1] * len(list2), list2, label="Data Points", alpha=0.2, color='green')
    axes[1].scatter(1, overall_average2, color='red', label="Overall Average")
    axes[1].set_title(title2)

    axes[1].set_xticks([])
    axes[1].legend()

    # Add a box with statistics for the second list
    stats_text2 = f"Average: {overall_average2:.3f}\nStd Dev: {std_dev2:.3f}"
    axes[1].text(0.7, 0.9, stats_text2, transform=axes[1].transAxes,
                 fontsize=12, verticalalignment='top', bbox=dict(boxstyle="round", facecolor="white", alpha=0.5))


    # Add an overall title and show the plot
    fig.suptitle("Vergleich Baseline Prompt mit optimiertem Prompt", fontsize=16)

    fig.text(0.5, 0.02, "15 Durchläufe mit je 380 Prompts",
             ha='center', fontsize=12, bbox=dict(boxstyle="round", facecolor="white", alpha=0.5))
    plt.show()


def get_accuracy_int(result):
    y_pred = result[2][0]
    y_true = result[2][1]

    accuracy = accuracy_score(y_true, y_pred)

    return round(accuracy * 100)

def get_accuracy(result):

    y_pred = result[1][0]
    y_true = result[1][1]

    accuracy = accuracy_score(y_true, y_pred)

    return accuracy * 100

def plot_confusion_matrix(result, y_pred, y_true):
    # Define the labels and calculate the confusion matrix
    labels = [1, 2, 3, 4, 5]
    cm = confusion_matrix(y_true, y_pred, labels=labels)
    # Convert to DataFrame for easier visualization
    cm_df = pd.DataFrame(cm, index=labels, columns=labels)
    # Plot the heatmap
    plt.figure(figsize=(8, 6))
    sns.heatmap(cm_df, annot=True, fmt="d", cmap="Blues", cbar=True, xticklabels=labels, yticklabels=labels, vmin=0,
                vmax=80)
    plt.xlabel(f"Predicted Labels")
    plt.ylabel("True Labels")
    plt.title(f"Confusion Matrix Heatmap {result[1]}")
    plt.show()
